/* Storage.qml
 *
 * Copyright 2013-2014 Jamie Strandboge <jamie@ubuntu.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import QtQuick 2.0
import "jsonpath.js" as JSONPath
import "util.js" as Util
import "build-trie.js" as BuildTrie
import U1db 1.0 as U1db

Item {
    id: storage

    // Setup the database
    U1db.Database {
        id: db
        path: "blabble.u1db"
    }

    // Store this separately which will make it easier to do db updates in the
    // future. These might look like:
    //   if (db_version != db_version_latest) {
    //     settings.contents = tmp // we could be a lot smarter about this
    //                             // and just update the individual keys
    //   }
    property var db_defaults: {
        "db_version": "1.0",
        "theme": {
            "current": "Ubuntu"
        },
        "rules": {
            "gridSize": "4",
            "time": "Unlimited",
            "minimumLength": "3"
        },
        "dictionary": {
            "primary": "Blabble",
            "secondary": null
        }
    }

    // don't store these in UIdb yet since we don't have a way to change them,
    // and if we did want to change them now, storing them in the db would make
    // it more difficult than needed
    property var themes: {
        "Ubuntu": { // http://design.ubuntu.com/brand/colour-palette
            "headerColor": "#2C001E",
            "backgroundColor": "#2C001E",
            "footerColor": "#361A36",
            "tileColor": "#77216F",
            "tileGradientColor": "#67115F",
            "textColor": "#FFFFFF",
            "hiTileColor": "#AEA79F",
            "hiTileGradientColor": "#6E675F",
            "hiTextColor": "#FFFFFF",
            "correctColor": "#1E9029",
            "correctGradientColor": "#0E6009",
            "correctTextColor": "#FFFFFF",
            "incorrectColor": "#DD4814",
            "incorrectGradientColor": "#BD2804",
            "incorrectTextColor": "#2C001E",
            "seenColor": "#3E59B0",
            "seenGradientColor": "#2E3990",
            "seenTextColor": "#FFFFFF"
        },
        "Ubuntu Retro": { // http://fc00.deviantart.net/fs29/i/2008/087/4/a/DVD_Ubuntu_8_04_brown__psd__by_elecete.jpg
            "headerColor": "#5A3B26",
            "backgroundColor": "#5A3B26",
            "footerColor": "#553620",
            "tileColor": "#B86929",
            "tileGradientColor": "#B86929",
            "textColor": "#FFFFFF",
            "hiTileColor": "#736C69",
            "hiTileGradientColor": "#736C69",
            "hiTextColor": "#FFFFFF",
            "correctColor": "#B8A48F",
            "correctGradientColor": "#B8A48F",
            "correctTextColor": "#FFFFFF",
            "incorrectColor": "#993B2A",
            "incorrectGradientColor": "#993B2A",
            "incorrectTextColor": "#FFFFFF",
            "seenColor": "#8FA3B8",
            "seenGradientColor": "#8FA3B8",
            "seenTextColor": "#FFFFFF"
        },
        "Green": {
            "headerColor": "#062808",
            "backgroundColor": "#062808",
            "footerColor": "#074020",
            "tileColor": "#1e9029",
            "tileGradientColor": "#0e7019",
            "textColor": "#FFFFFF",
            "hiTileColor": "#30e342",
            "hiTileGradientColor": "#10a322",
            "hiTextColor": "#0c3810",
            "correctColor": "#0c3810",
            "correctGradientColor": "#0c2805",
            "correctTextColor": "#FFFFFF",
            "incorrectColor": "#8f291e",
            "incorrectGradientColor": "#6f1906",
            "incorrectTextColor": "#000000",
            "seenColor": "#0C1E38",
            "seenGradientColor": "#0C0E28",
            "seenTextColor": "#FFFFFF"
        }
    }

    // Declare a document
    U1db.Document {
        database: db
        docId: "settings"
        id: settings
        create: true
        defaults: db_defaults
    }


    property var json
    property string query: "$"
    property string dictVersion: ""
    property string dictName: ""
    property var dictAlphabet: {}
    property var dictWords: []
    property var dictTrie: null
    property string dictPrimary: ""
    property string dictSecondary: ""

    signal dictInitialized()

    // FIXME: this whole thing could be moved out to another C++ thread
    // (WorkerScript is too slow to copy the data around)
    // Alternatively, save the trie in the U1db and update it only if the
    // dictionary version changes
    function loadDictionary() {
        // TODO: combine primary and secondary
        if (dictPrimary === "") {
            dictPrimary = _get_dictionary_path("primary", getSetting("dictionary", "primary"))
        }

        Util.debug("Loading " + dictPrimary)
        var xhr = new XMLHttpRequest;
        xhr.open("GET", dictPrimary);
        xhr.onreadystatechange = function() {
            if (xhr.readyState == XMLHttpRequest.DONE) {
                json = xhr.responseText;
                //Util.debug("JSON: " + json)
                if (json !== "") {
                    dictAlphabet = null
                    dictWords = null
                    dictTrie = null
                    dictAlphabet = {}
                    dictWords = []
                    dictTrie = {}

                    var objectArray = parseJSONString(json, query);

                    for ( var key in objectArray ) {
                        var jo = objectArray[key];

                        dictName = jo['name']
                        dictVersion = jo['version']
                        dictAlphabet = jo['alphabet']
                        dictWords = jo['words']
                        //for (var i = 0; i < dictWords.length; i++) {
                        //    Util.debug("" + dictWords[i])
                        //}
                    }

                    dictTrie = BuildTrie.buildTrie(dictWords)

                    dictInitialized()
                    Util.debug("Initialized dictionary")
                    Util.debug(" word count = " + dictWords.length)
                    Util.debug(" trie top nodes = " +
                               Object.keys(dictTrie).length)
                }
            }
        }
        xhr.send();
    }

    function parseJSONString(jsonString, jsonPathQuery) {
        var objectArray = JSON.parse(jsonString);
        if ( jsonPathQuery !== "" )
            objectArray = JSONPath.jsonPath(objectArray, jsonPathQuery);

        return objectArray;
    }

    function getSetting(type, name) {
        // themes are handled in a separate dictionary for now
        if (name !== "current" && type === "theme") {
            var current = settings.contents[type]["current"]
            return themes[current][name]
        }
        return settings.contents[type][name]
    }

    function _get_dictionary_path(name, value) {
        var path = ""
        if (name === "primary") {
            switch(value) {
            case "Blabble":
                path = Qt.resolvedUrl("../wordlists/blabble.json")
                break
            case "ENABLE":
                path = Qt.resolvedUrl("../wordlists/enable_expurgated.json")
                break
            case "TWL06":
                path = Qt.resolvedUrl("../wordlists/twl06_expurgated.json")
                break
            }
        }
        return path
    }

    function setSetting(type, name, value) {
        // for some reason, we have to copy the whole database then copy back
        // instead of just doing: settings.contents[type][name] = value
        // This is confirmed with upstream U1db
        var tmp = settings.contents
        tmp[type][name] = value
        settings.contents = tmp

        if (type === "dictionary") {
            dictPrimary = _get_dictionary_path(name, value)
            loadDictionary()
        }
    }

    function getItems(type, sub) {
        var keys = []
        // themes are handled in a separate dictionary for now
        if (type === "theme") {
            for (var key in themes) {
                keys.push(key)
            }
            keys.sort()
        } else if (type === "dictionary" && sub === "primary") {
            keys.push("Blabble")
            keys.push("ENABLE")
            keys.push("TWL06")
            keys.sort()
        } else if (type === "rules" && sub === "gridSize") {
            // ordered for simple arithmetic
            keys.push("4x4")
            keys.push("5x5")
            keys.push("6x6")
            keys.sort()
        } else if (type === "rules" && sub === "minimumLength") {
            // ordered for simple arithmetic
            keys.push("3")
            keys.push("4")
            keys.push("5")
            keys.push("6")
            keys.sort()
        } else if (type === "rules" && sub === "time") {
            // ordered for simple arithmetic
            keys.push("Unlimited")
            keys.push("2 minutes")
            keys.push("3 minutes")
            keys.push("5 minutes")
            keys.push("10 minutes")
            keys.push("15 minutes")
        } else {
            for (var key in settings.contents[type]) {
                if (key !== "current" &&
                    settings.contents[type].hasOwnProperty(key)) {
                    keys.push(key)
                }
            }
            keys.sort()
        }
        return keys
    }
}
