/* SettingsTab.qml
 *
 * Copyright 2013-2014 Jamie Strandboge <jamie@ubuntu.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1
import "../components"
import "../components/util.js" as Util

Tab {
    title: i18n.tr("Settings")

    page: Page {
        anchors.fill: parent
        width: parent.width
        anchors.margins: units.gu(2)

        Flickable {
            id: myFlickable
            anchors.fill: parent
            contentHeight: settingsColumn.height

            Column {
                id: settingsColumn
                width: page.width
                spacing: units.gu(3)

                // Settings sections: put each section into its own column so
                // we can have different spacing between sections and items
                // within the section

                // Theme settings
                Column {
                    spacing: units.gu(1)
                    width: parent.width
                    Label {
                        text: i18n.tr("Theme")
                        fontSize: "large"
                    }
                    ListItem.ItemSelector {
                        id: themeSelector
                        property var themes: storage.getItems("theme", "")
                        expanded: false
                        selectedIndex: themes.indexOf(storage.getSetting("theme", "current"))
                        model: themes
                        onSelectedIndexChanged: {
                            Util.debug("theme changed to " +
                                       themes[themeSelector.selectedIndex])
                            storage.setSetting("theme", "current",
                                    themes[themeSelector.selectedIndex])

                            mainView.headerColor = storage.getSetting("theme", "headerColor")
                            mainView.backgroundColor = storage.getSetting("theme", "backgroundColor")
                            mainView.footerColor = storage.getSetting("theme", "footerColor")
                            mainView.game.updateColor()
                        }
                    }
                }

                // dictionary settings
                Column {
                    id: dictionarySection
                    spacing: units.gu(1)
                    width: parent.width
                    property var lastSelection: 0

                    Label {
                        text: i18n.tr("Dictionaries")
                        fontSize: "large"
                    }
                    ListItem.ValueSelector {
                        id: dictSelector
                        expanded: false
                        text: i18n.tr("Primary")
                        values: storage.getItems("dictionary", "primary")
                        selectedIndex: values.indexOf(storage.getSetting("dictionary", "primary"))
                        onSelectedIndexChanged: {
                            if (dictionarySection.lastSelection !== dictSelector.selectedIndex) {
                                if (gameState.playing === true) {
                                    PopupUtils.open(dictConfirmDialog, null)
                                } else {
                                    dictionarySection._change_dictionary()
                                }
                            }
                        }
                    }

                    function _change_dictionary() {
                        dictionarySection.lastSelection = dictSelector.selectedIndex

                        // Setting the dictionary calls loadDictionary() which
                        // sends a signal which mainView.storage uses to call
                        // createGameBoard(), so we don't have to do anything
                        // special here.
                        storage.setSetting("dictionary", "primary",
                                           dictSelector.values[dictSelector.selectedIndex])
                        Util.debug("dict changed to " + dictSelector.selectedIndex)
                    }

                    Component {
                        id: dictConfirmDialog

                        Dialog {
                            id: dialog
                            title: i18n.tr("Change dictionary")
                            text: i18n.tr("Changing the dictionary will end the current game and reset the board. Are you sure you want to change the dictionary now?")
                            Button {
                                text: i18n.tr("Cancel")
                                gradient: UbuntuColors.greyGradient
                                onClicked: {
                                    PopupUtils.close(dialog)
                                    dictSelector.selectedIndex = dictionarySection.lastSelection
                                    Util.debug("dictionary changed back to " + dictionarySection.lastSelection)
                                }
                            }
                            Button {
                                text: i18n.tr("Change dictionary")
                                onClicked: {
                                    PopupUtils.close(dialog)
                                    dictionarySection._change_dictionary()
                                }
                            }
                        }
                    }
                }

                // rules settings
                Column {
                    spacing: units.gu(1)
                    width: parent.width
                    id: rulesSection

                    Label {
                        text: i18n.tr("Rules")
                        fontSize: "large"
                    }

                    // gridSize
                    property var gridSizeSelection: storage.getSetting("rules", "gridSize")

                    ListItem.ValueSelector {
                        id: gridSizeSelector
                        expanded: false
                        text: i18n.tr("Grid size")

                        values: storage.getItems("rules", "gridSize")
                        selectedIndex: values.indexOf(storage.getSetting("rules", "gridSize") +
                                       "x" + storage.getSetting("rules", "gridSize"))

                        onSelectedIndexChanged: {
                            // Make this a string
                            var sel = "" + (gridSizeSelector.selectedIndex + 4)
                            if (sel !== storage.getSetting("rules", "gridSize")) {
                                if (gameState.playing === true) {
                                    PopupUtils.open(gridSizeConfirmDialog, null)
                                } else {
                                    rulesSection._change_grid_size()
                                }
                            }
                        }
                    }

                    function _change_grid_size() {
                        rulesSection.gridSizeSelection = "" + (gridSizeSelector.selectedIndex + 4)
                        Util.debug("gridSize changed to " + rulesSection.gridSizeSelection)
                        storage.setSetting("rules", "gridSize", rulesSection.gridSizeSelection)
                        mainView.game.createGameBoard()
                    }

                    Component {
                        id: gridSizeConfirmDialog

                        Dialog {
                            id: dialog
                            title: i18n.tr("Adjust grid size")
                            text: i18n.tr("Changing the grid size will end the current game and reset the board to the new size. Are you sure you want to change the grid size now?")
                            Button {
                                text: i18n.tr("Cancel")
                                gradient: UbuntuColors.greyGradient
                                onClicked: {
                                    PopupUtils.close(dialog)
                                    gridSizeSelector.selectedIndex = (rulesSection.gridSizeSelection - 4)
                                    Util.debug("gridSizeSelector changed back to " + gridSizeSelector.selectedIndex)
                                }
                            }
                            Button {
                                text: i18n.tr("Change grid size")
                                onClicked: {
                                    PopupUtils.close(dialog)
                                    rulesSection._change_grid_size()
                                }
                            }
                        }
                    }

                    // minimumLength
                    property var minimumLengthSelection: storage.getSetting("rules", "minimumLength")

                    ListItem.ValueSelector {
                        id: minimumLengthSelector
                        expanded: false
                        text: i18n.tr("Minimum word length")
                        values: storage.getItems("rules", "minimumLength")
                        selectedIndex: values.indexOf(storage.getSetting("rules", "minimumLength"))

                        onSelectedIndexChanged: {
                            // Make this a string
                            var sel = "" + (minimumLengthSelector.selectedIndex + 3)
                            if (sel !== storage.getSetting("rules", "minimumLength")) {
                                if (gameState.playing === true) {
                                    PopupUtils.open(minimumLengthConfirmDialog, null)
                                } else {
                                    rulesSection._change_minimum_length()
                                }
                            }
                        }
                    }

                    function _change_minimum_length() {
                        rulesSection.minimumLengthSelection = "" + (minimumLengthSelector.selectedIndex + 3)
                        Util.debug("minimumLength changed to " + rulesSection.minimumLengthSelection)
                        storage.setSetting("rules", "minimumLength", rulesSection.minimumLengthSelection)
                        mainView.game.createGameBoard()
                    }

                    Component {
                        id: minimumLengthConfirmDialog

                        Dialog {
                            id: dialog
                            title: i18n.tr("Adjust minimum word length")
                            text: i18n.tr("Changing the minimum word length will end the current game and reset the board. Are you sure you want to change the minimum word length now?")
                            Button {
                                text: i18n.tr("Cancel")
                                gradient: UbuntuColors.greyGradient
                                onClicked: {
                                    PopupUtils.close(dialog)
                                    minimumLengthSelector.selectedIndex = (rulesSection.minimumLengthSelection - 3)
                                    Util.debug("minimumLengthSelector changed back to " + minimumLengthSelector.selectedIndex)
                                }
                            }
                            Button {
                                text: i18n.tr("Change word length")
                                onClicked: {
                                    PopupUtils.close(dialog)
                                    rulesSection._change_minimum_length()
                                }
                            }
                        }
                    }

                    // timer
                    ListItem.ValueSelector {
                        id: timerSelector
                        expanded: false
                        text: i18n.tr("Time")
                        values: storage.getItems("rules", "time")
                        selectedIndex: values.indexOf(storage.getSetting("rules", "time"))
                        onSelectedIndexChanged: {
                            Util.debug("timer changed to " + values[timerSelector.selectedIndex])
                            storage.setSetting("rules", "time", values[timerSelector.selectedIndex])
                        }
                    }
                }
            }
        }
    }
}
